<?php namespace EscapeWork\Manager\Medias\Helpers;

use EscapeWork\Manager\Medias\Models\MediaModel;
use EscapeWork\Resize\Resize;
use Symfony\Component\HttpFoundation\File\UploadedFile;
use Illuminate\Support\MessageBag;
use Str;

class ImageUploader
{

    /**
     * Directory
     */
    protected $dir;

    /**
     * MessageBag
     */
    public $messageBag;

    /**
     * MediaModel
     */
    protected $model;

    /**
     * Filename
     */
    public $filename;

    /**
     * Tipos de imagens aceitas
     */
    protected $acceptedImages = array(
        'image/jpg', 'image/jpeg', 'image/pjpg', 'image/pjpeg',
        'image/png'
    );

    public function __construct($dir, $model)
    {
        $this->dir   = $dir;
        $this->model = $model;

        $this->messageBag = new MessageBag();
    }

    /**
     * @return boolean
     */
    public function upload($image, $update = false, $oldImage)
    {
        if (! $image instanceof UploadedFile) {
            return false;
        }

        # validando o tipo
        if (! $this->_validateType($image->getMimeType())) {
            $this->messageBag->add('errors', 'Apenas imagens <strong>jpg e png</strong> são aceitas');

            return false;
        }

        # formatando o nome
        $dir      = $this->getDir();
        $filename = $this->_formatName($image->getClientOriginalName(), $dir, $image->guessExtension());

        # copiando a imagem
        try {
            $image = $image->move($dir . '/', $filename);
            $this->createSizes($dir, $filename);

            # excluindo o arquivo atual
            if ($update) {
                $this->_deleteImage($oldImage);
            }

            $this->filename = $filename;

            return true;
        } catch (Exception $e) {
            return false;
        }
    }

    protected function getDir()
    {
        return $this->dir;
    }

    public function _validateType($type)
    {
        return in_array($type, $this->acceptedImages);
    }

    protected function _formatName($name, $dir, $extension)
    {
        $name    = explode('.', $name);
        $total   = count($name);

        if ($total > 1) {
            unset($name[$total - 1]);
        }

        $name     = implode('', $name);
        $filename = Str::slug($name) . '.' . $extension;

        while (is_file($dir . '/' . $filename)) {
            $filename = '_' . $filename;
        }

        return $filename;
    }

    protected function createSizes($dir, $filename)
    {
        if (! $sizes = $this->getSizes()) {
            $this->messageBag->add('errors', 'Não foi possível buscar os tamanhos da imagem');
            return false;
        }

        foreach ($sizes as $size) {
            $prefix = $size->prefix == 'default' ? '' : $size->prefix;
            $banner = $dir . '/' . $prefix . $filename;

            # copiando o arquivo
            if (! is_file($banner)) {
                copy($dir . '/' . $filename, $banner);
            }

            # resize
            $resize = new Resize($banner);
            $resize->setWidth($size->width)->setHeight($size->height);

            if ($size->resize) {
                $resize->resize();
            } else {
                $resize->crop();
            }
        }
    }

    protected function getSizes()
    {
        if (! $mediaModel = MediaModel::where('model', '=', $this->model)->first()) {
            return false;
        }

        return $mediaModel->sizes;
    }

    public function _deleteImage($oldImage)
    {
        $file = $this->getDir() . '/' . $oldImage;

        if (is_file($file)) {
            unlink($file);
        }
    }
}
